/* TaskExample03.ino
 * Improvement on TaskExample02.
 * Moved functionality into seperate functions.
 */

int onThreshold = 700;
int offThreshold = 500;

int ledPin = 13;

int rPin = 5;
int gPin = 9;
int bPin = 6;

int outputPin = 7;

int analogPin = A0;

int loopCount = 0;
int ledState = LOW;

int analogValue;

void setup(void)
{
  pinMode(ledPin, OUTPUT);
  pinMode(rPin, OUTPUT);
  pinMode(gPin, OUTPUT);
  pinMode(bPin, OUTPUT);
  pinMode(outputPin, OUTPUT);

  pinMode(analogPin, INPUT);

  digitalWrite(ledPin, LOW);
  digitalWrite(outputPin, LOW);

  Serial.begin(115200);
}

void loop(void)
{
  /* We want the LED to blink every 500 ms */
  if (loopCount % 5 == 0)
  {
    blinkLED();
  }

  /* We want to read the analog voltage every 100ms */
  readAnalogVoltage();

  /* We want to check for new serial data every 200ms */
  if (loopCount % 2 == 0)
  {
    readSerialAndUpdateRGB();
  }

  /* We want to send the latest analog reading back every 1000ms*/
  if (loopCount % 10 == 0)
  {
    sendReading();
  }

  delay(100);
  updateLoopCount();
}

void blinkLED(void)
{
  ledState = !ledState; // HIGH -> LOW, LOW->HIGH
  digitalWrite(ledPin, ledState);
}

void readAnalogVoltage(void)
{
  analogValue = analogRead(analogPin);
  if (analogValue < offThreshold)
  {
    digitalWrite(outputPin, LOW);
  }
  else if (analogValue > onThreshold)
  {
    digitalWrite(outputPin, HIGH);
  }
}

void readSerialAndUpdateRGB(void)
{
  if (Serial.available() > 0)
  {
    int r = Serial.parseInt();
    int g = Serial.parseInt();
    int b = Serial.parseInt();

    analogWrite(rPin, r);
    analogWrite(gPin, g);
    analogWrite(bPin, b);

    // Discard any other characters
    while (Serial.available() > 0) { 
      Serial.read(); 
    }

  }
}

void sendReading(void)
{
  Serial.print("Analog read: ");
  Serial.println(analogValue);
}

void updateLoopCount(void)
{
  loopCount++;
  if (loopCount == 10)
  {
    // Ten times round the loop is one second, reset the counter
    loopCount = 0;
  }
}

