/* TaskExample04.ino
 * Improvement on TaskExample03.
 * Uses millis() function for timing
 */

int onThreshold = 700;
int offThreshold = 500;

int ledPin = 13;

int rPin = 5;
int gPin = 9;
int bPin = 6;

int outputPin = 7;

int analogPin = A0;

int ledState = LOW;

int analogValue;

int oldMillisLed = 0;
int oldMillisRGB = 0;
int oldMillisAnalog = 0;
int oldMillisSerial = 0;

void setup(void)
{
  pinMode(ledPin, OUTPUT);
  pinMode(rPin, OUTPUT);
  pinMode(gPin, OUTPUT);
  pinMode(bPin, OUTPUT);
  pinMode(outputPin, OUTPUT);

  pinMode(analogPin, INPUT);

  digitalWrite(ledPin, LOW);
  digitalWrite(outputPin, LOW);

  Serial.begin(115200);
}

void loop(void)
{
  int currentMillis = millis();

  /* We want the LED to blink every 500 ms */
  if (currentMillis - oldMillisLed > 500)
  {
    oldMillisLed = currentMillis;
    blinkLED();
  }

  /* We want to read the analog voltage every 100ms */
  if (currentMillis - oldMillisAnalog > 100)
  {
    oldMillisAnalog = currentMillis;
    readAnalogVoltage();
  }

  /* We want to check for new serial data every 200ms */
  if (currentMillis - oldMillisRGB > 200)
  {
    oldMillisRGB = currentMillis;
    readSerialAndUpdateRGB();
  }

  /* We want to send the latest analog reading back every 1000ms*/
  if (currentMillis - oldMillisSerial > 1000)
  {
    oldMillisSerial = currentMillis;
    sendReading();
  }
}

void blinkLED(void)
{
  ledState = !ledState; // HIGH -> LOW, LOW->HIGH
  digitalWrite(ledPin, ledState);
}

void readAnalogVoltage(void)
{
  analogValue = analogRead(analogPin);
  if (analogValue < offThreshold)
  {
    digitalWrite(outputPin, LOW);
  }
  else if (analogValue > onThreshold)
  {
    digitalWrite(outputPin, HIGH);
  }
}

void readSerialAndUpdateRGB(void)
{
  if (Serial.available() > 0)
  {
    int r = Serial.parseInt();
    int g = Serial.parseInt();
    int b = Serial.parseInt();

    analogWrite(rPin, r);
    analogWrite(gPin, g);
    analogWrite(bPin, b);

    // Discard any other characters
    while (Serial.available() > 0) { 
      Serial.read(); 
    }

  }
}

void sendReading(void)
{
  Serial.print("Analog read: ");
  Serial.println(analogValue);
}




