/* TaskExample05.ino
 * Improvement on TaskExample04.
 * Uses task library
 */

#include <TaskAction.h>

#define LED_TICK_MS 500
#define ANALOG_READ_TICK_MS 100
#define SERIAL_READ_TICK_MS 200
#define SERIAL_OUTPUT_TICK_MS 1000

int onThreshold = 700;
int offThreshold = 500;

int ledPin = 13;

int rPin = 5;
int gPin = 9;
int bPin = 6;

int outputPin = 7;

int analogPin = A0;

int ledState = LOW;

int analogValue;

TaskAction blinkLEDTask = TaskAction(blinkLED, LED_TICK_MS, INFINITE_TICKS);
TaskAction readAnalogTask = TaskAction(readAnalogVoltage, ANALOG_READ_TICK_MS, INFINITE_TICKS);
TaskAction readSerialTask = TaskAction(readSerialAndUpdateRGB, SERIAL_READ_TICK_MS, INFINITE_TICKS);
TaskAction sendSerialTask = TaskAction(sendReading, SERIAL_OUTPUT_TICK_MS, INFINITE_TICKS);

void setup(void)
{
  pinMode(ledPin, OUTPUT);
  pinMode(rPin, OUTPUT);
  pinMode(gPin, OUTPUT);
  pinMode(bPin, OUTPUT);
  pinMode(outputPin, OUTPUT);

  pinMode(analogPin, INPUT);

  digitalWrite(ledPin, LOW);
  digitalWrite(outputPin, LOW);

  Serial.begin(115200);
}

void loop(void)
{
  blinkLEDTask.tick();
  readAnalogTask.tick();
  readSerialTask.tick();
  sendSerialTask.tick();
}

void blinkLED(void)
{
  ledState = !ledState; // HIGH -> LOW, LOW->HIGH
  digitalWrite(ledPin, ledState);
}

void readAnalogVoltage(void)
{
  analogValue = analogRead(analogPin);
  if (analogValue < offThreshold)
  {
    digitalWrite(outputPin, LOW);
  }
  else if (analogValue > onThreshold)
  {
    digitalWrite(outputPin, HIGH);
  }
}

void readSerialAndUpdateRGB(void)
{
  if (Serial.available() > 0)
  {
    int r = Serial.parseInt();
    int g = Serial.parseInt();
    int b = Serial.parseInt();

    analogWrite(rPin, r);
    analogWrite(gPin, g);
    analogWrite(bPin, b);

    // Discard any other characters
    while (Serial.available() > 0) { 
      Serial.read(); 
    }

  }
}

void sendReading(void)
{
  Serial.print("Analog read: ");
  Serial.println(analogValue);
}

